import requests
import datetime
import base64
from requests.auth import HTTPBasicAuth

class MpesaPayments:
    def __init__(self, businessshortCode, lipa_na_mpesa_pass_key, consumer_secret_key, consumer_key, call_back_url, account_reference, transaction_desc):
        self.businessshortCode = businessshortCode
        self.lipa_na_mpesa_pass_key = lipa_na_mpesa_pass_key
        self.consumer_secret_key = consumer_secret_key
        self.consumer_key = consumer_key
        self.call_back_url = call_back_url
        self.account_reference = account_reference
        self.transaction_desc = transaction_desc

    def generate_timestamp(self):
        unformated_time = datetime.datetime.now()
        formated_time = unformated_time.strftime("%Y%m%d%H%M%S")
        return formated_time

    def generate_access_token(self):
        api_URL = "https://sandbox.safaricom.co.ke/oauth/v1/generate?grant_type=client_credentials"
        r = requests.get(api_URL, auth=HTTPBasicAuth(self.consumer_key, self.consumer_secret_key))
        json_response = r.json()
        my_access_token = json_response["access_token"]
        return my_access_token

    def generate_password(self):
        data_to_encode = self.businessshortCode + self.lipa_na_mpesa_pass_key + self.generate_timestamp()
        encoded_string = base64.b64encode(data_to_encode.encode())
        decoded_password = encoded_string.decode('utf8')
        return decoded_password

    def lipa_na_mpesa_online(self, phone_number, amount):
        access_token = self.generate_access_token()
        api_url = "https://sandbox.safaricom.co.ke/mpesa/stkpush/v1/processrequest"
        headers = {
            "Authorization": f"Bearer {access_token}",
            "Content-Type": "application/json"
        }
        request = {
            "BusinessShortCode": self.businessshortCode,
            "Password": self.generate_password(),
            "Timestamp": self.generate_timestamp(),
            "TransactionType": "CustomerPayBillOnline",
            "Amount": amount,
            "PartyA": phone_number,
            "PartyB": self.businessshortCode,
            "PhoneNumber": phone_number,
            "CallBackURL": self.call_back_url,
            "AccountReference": self.account_reference,
            "TransactionDesc": self.transaction_desc,
        }
        response = requests.post(api_url, json=request, headers=headers)
        return response.json() if response.status_code == 200 else {'success': False, 'message': f'Failed to initiate payment: {response.text}'}

# # Example usage:
# mpesa_service = MpesaPayments(
#     businessshortCode="174379",
#     lipa_na_mpesa_pass_key="bfb279f9aa9bdbcf158e97dd71a467cd2e0c893059b10f78e6b72ada1ed2c919",
#     consumer_secret_key="SrlhfpZUudWdFnR8NzGSOv0ln2zOD95bC1Z0PLDN4hOVurSAcjb4cugrLxNVwb02",
#     consumer_key="34nHxWSAuEIHqLFDx4y5ynEoTvIrE6V7AAPrxdwFeoJj8ySa",
#     call_back_url="https://webhook.site/6cb0292e-6014-463b-8a38-c04146e9b14b",
#     account_reference="Order123",
#     transaction_desc="Payment for service"
# )

# result = mpesa_service.lipa_na_mpesa_online("254717713943", "1")
# print(result)
